unit KA.Utilities.CopyFile;
interface

uses
  System.IO,
  Borland.Vcl.SysConst,
  Borland.Vcl.Consts,
  Borland.Vcl.Types,
  Borland.Vcl.Windows,
  Borland.Vcl.Messages,
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.Forms,
  Borland.Vcl.Dialogs;

type
  TCopyType          = (All, Folders, Files, None);
  TCopyResult        = (Copyed, Aborted, Ignored, Skipped);
  TOnProgressEvent   = Procedure(SourceFileName, TargetFileName : String; Written, Total : Double; Var Abort : Boolean) of Object;
  TOnBeforeCopyEvent = Procedure(Var SourceFileName  : String; Var TargetFileName : String; Var Abort : Boolean; Var DoCopy : Boolean) of Object;
  TOnAfterCopyEvent  = Procedure(SourceFileName, TargetFileName : String) of Object;
  TOnErrorEvent      = Procedure(SourceFileName, TargetFileName : String; Var Abort : Boolean; Var Retry : Boolean; Var Ignore : Boolean) of Object;
  TOnRenameEvent     = Procedure(Var SourceFileName  : String; Var TargetFileName : String) of Object;

  TKACopyFile = class(TComponent)
  private
    FWhatToCopy            : TCopyType;
    FBufferSize            : Integer;
    FCopyAttributes        : Boolean;
    FCopyDates             : Boolean;
    FCopySubfolders        : Boolean;
    FTarget                : String;
    FSource                : String;
    FFailIfExists          : Boolean;
    FRenameIfExists        : Boolean;
    FBytesRead             : Cardinal;
    FBytesWritten          : Cardinal;
    FBytesTotal            : Cardinal;
    FFilesTotal            : Cardinal;
    FFoldersTotal          : Cardinal;
    FFilesCurrent          : Cardinal;
    FBytesCurrent          : Cardinal;
    FFoldersCurrent        : Cardinal;
    FActivate              : Boolean;
    FYeld                  : Boolean;
    FOnAfterCopy           : TOnAfterCopyEvent;
    FOnBeforeCopy          : TOnBeforeCopyEvent;
    FOnError               : TOnErrorEvent;
    FOnProgress            : TOnProgressEvent;
    FDirWildCard           : String;
    FFileWildCard          : String;
    FOnAfterCopyFolder     : TOnAfterCopyEvent;
    FOnBeforeCopyFolder    : TOnBeforeCopyEvent;
    FOnAutoReaname         : TOnRenameEvent;



    procedure SetWhatToCopy(const Value: TCopyType);
    procedure SetBufferSize(const Value: Integer);
    procedure SetCopySubfolders(const Value: Boolean);
    procedure SetSource(const Value: String);
    procedure SetTarget(const Value: String);
    procedure SetFailIfExists(const Value: Boolean);
    procedure SetRenameIfExists(const Value: Boolean);
    procedure SetActivate(const Value: Boolean);
    procedure SetYeld(const Value: Boolean);
    procedure SetOnAfterCopy(const Value: TOnAfterCopyEvent);
    procedure SetOnBeforeCopy(const Value: TOnBeforeCopyEvent);
    procedure SetOnError(const Value: TOnErrorEvent);
    procedure SetOnProgress(const Value: TOnProgressEvent);
    procedure SetDirWildCard(const Value: String);
    procedure SetFileWildCard(const Value: String);
    procedure SetCopyDates(const Value: Boolean);
    procedure SetCopyAttributes(const Value: Boolean);
    procedure SetOnAfterCopyFolder(const Value: TOnAfterCopyEvent);
    procedure SetOnBeforeCopyFolder(const Value: TOnBeforeCopyEvent);
    procedure SetBytesTotal(const Value: Cardinal);
    procedure SetOnAutoReaname(const Value: TOnRenameEvent);
    procedure SetFilesTotal(const Value: Cardinal);
    procedure SetFoldersTotal(const Value: Cardinal);
    procedure SetBytesRead(const Value: Cardinal);
    procedure SetBytesWritten(const Value: Cardinal);
    procedure SetBytesCurrent(const Value: Cardinal);
    procedure SetFilesCurrent(const Value: Cardinal);
    procedure SetFoldersCurrent(const Value: Cardinal);
    { Private declarations }
  protected
    { Protected declarations }
    Function    TrailingSlash(DirectoryName:String):String;
    Function    UnTrailingSlash(DirectoryName:String):String;
    Function    GetUniqueName(FileName:String):String;
    Function    DoCopyFile(SourceFileName : String; TargetFileName:String):TCopyResult;
    Function    CopyDirectory(SourceDirName : String; TargetDirName:String; Var Abort  : Boolean):TCopyResult;
    Function    SetFolderTime(FolderName:String; Dir: TSearchRec):Boolean;
    Function    SetFileTime(FileName:String; Dir: TSearchRec):Boolean;
    Function    SetAttributes(FileName:String; Dir: TSearchRec):Boolean;
  public
    { Public declarations }
    Function    Run:TCopyResult;
    Function    CopyFile(Src:String; Dest:String):TCopyResult;
    Constructor Create(AOwner:TComponent);Override;
    Destructor  Destroy;Override;

    Property    BytesRead      : Cardinal read FBytesRead write SetBytesRead;
    Property    BytesWritten   : Cardinal read FBytesWritten write SetBytesWritten;

    Property    BytesTotal     : Cardinal read FBytesTotal write SetBytesTotal;
    Property    FilesTotal     : Cardinal read FFilesTotal write SetFilesTotal;
    Property    FoldersTotal   : Cardinal read FFoldersTotal write SetFoldersTotal;

    Property    BytesCurrent   : Cardinal read FBytesCurrent write SetBytesCurrent;
    Property    FilesCurrent   : Cardinal read FFilesCurrent write SetFilesCurrent;
    Property    FoldersCurrent : Cardinal read FFoldersCurrent write SetFoldersCurrent;

  published
    { Published declarations }
    Property WhatToCopy          : TCopyType          read FWhatToCopy         write SetWhatToCopy;
    Property CopyAttributes      : Boolean            read FCopyAttributes     write SetCopyAttributes;
    Property CopyDates           : Boolean            read FCopyDates          write SetCopyDates;
    Property CopySubfolders      : Boolean            read FCopySubfolders     write SetCopySubfolders;
    Property BufferSize          : Integer            read FBufferSize         write SetBufferSize;
    Property Source              : String             read FSource             write SetSource;
    Property Target              : String             read FTarget             write SetTarget;
    Property FileWildCard        : String             read FFileWildCard       write SetFileWildCard;
    Property FolderWildCard      : String             read FDirWildCard        write SetDirWildCard;
    Property FailIfExists        : Boolean            read FFailIfExists       write SetFailIfExists;
    Property RenameIfExists      : Boolean            read FRenameIfExists     write SetRenameIfExists;
    Property Yeld                : Boolean            read FYeld               write SetYeld;
    Property Activate            : Boolean            read FActivate           write SetActivate;
    Property OnProgress          : TOnProgressEvent   read FOnProgress         write SetOnProgress;
    Property OnBeforeCopyFile    : TOnBeforeCopyEvent read FOnBeforeCopy       write SetOnBeforeCopy;
    Property OnAfterCopyFile     : TOnAfterCopyEvent  read FOnAfterCopy        write SetOnAfterCopy;
    Property OnBeforeCopyFolder  : TOnBeforeCopyEvent read FOnBeforeCopyFolder write SetOnBeforeCopyFolder;
    Property OnAfterCopyFolder   : TOnAfterCopyEvent  read FOnAfterCopyFolder  write SetOnAfterCopyFolder;
    Property OnCopyError         : TOnErrorEvent      read FOnError            write SetOnError;
    Property OnAutoReaname       : TOnRenameEvent     read FOnAutoReaname      write SetOnAutoReaname;
  end;

procedure Register;

implementation

{$R 'KA.Utilities.CopyFile.TKACopyFile.bmp'}
procedure Register;
begin
  RegisterComponents('KA', [TKACopyFile]);
end;

{ TKACopyFile }

Function TKACopyFile.SetFolderTime(FolderName:String; Dir: TSearchRec):Boolean;
Var
  F      : THandle;
  DummyH : THandle;
Begin
  Result := False;
  DummyH := 0;
  F := CreateFile(FolderName,GENERIC_WRITE,FILE_SHARE_WRITE or FILE_SHARE_DELETE,NIL,OPEN_EXISTING,FILE_FLAG_BACKUP_SEMANTICS,DummyH);
  if F <> INVALID_HANDLE_VALUE Then
     Begin
       Result := Borland.Vcl.Windows.SetFileTime(F,Dir.FindData.ftCreationTime,Dir.FindData.ftLastAccessTime,Dir.FindData.ftLastWriteTime);
       CloseHandle(F);
     End;
End;

Function  TKACopyFile.SetFileTime(FileName:String; Dir: TSearchRec):Boolean;
Var
  F      : THandle;
  DummyH : THandle;
Begin
  Result := False;
  DummyH := 0;
  F := CreateFile(FileName,GENERIC_WRITE,FILE_SHARE_WRITE,NIL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,DummyH);
  if F <> INVALID_HANDLE_VALUE Then
     Begin
       Result := Borland.Vcl.Windows.SetFileTime(F,Dir.FindData.ftCreationTime,Dir.FindData.ftLastAccessTime,Dir.FindData.ftLastWriteTime);
       CloseHandle(F);
     End;
End;

Function  TKACopyFile.SetAttributes(FileName:String; Dir: TSearchRec):Boolean;
Begin
  Result := SetFileAttributes(FileName,Dir.Attr);
End;


Function TKACopyFile.DoCopyFile(SourceFileName, TargetFileName: String):TCopyResult;
Var
 SF     : TFileStream;
 TF     : TFileStream;
 BUF    : TBytes;
 NR     : Integer;
 NW     : Integer;
 FS     : Double;
 D1     : Double;
 D2     : Double;
 D3     : Double;
 FT     : Cardinal;
 Abort  : Boolean;
 Retry  : Boolean;
 Ignore : Boolean;
 Res    : Integer;
 Dir    : TSearchRec;
begin
 {$I+}
 Result  := Copyed;
 FT      := 0;
 FS      := 0;
 NR      := 0;
 NW      := 0;
 SetLength(BUF,FBufferSize);
 Try
   Abort  := False;
   Retry  := False;
   Ignore := False;
   //***************************************************************************
   if FFailIfExists Then
      Begin
        if FileExists(TargetFileName) Then
         Try
           Raise Exception.Create('File: '+#13#10+TargetFileName+#13#10+' already exists!');
         Except
           if Assigned(FOnError) Then FOnError(SourceFileName, TargetFileName,Abort,Retry,Ignore);
           if Abort Then
              Begin
                Result := Aborted;
                Exit;
              End;
         End;
      End;
   //***************************************************************************
   if FRenameIfExists Then
      Begin
        if FileExists(TargetFileName) Then
           Begin
             TargetFileName := GetUniqueName(TargetFileName);
             if Assigned(FOnAutoReaname) Then FOnAutoReaname(SourceFileName,TargetFileName);
           End;
      End;
   //***************************************************************************
   SetErrorMode(SEM_FAILCRITICALERRORS);

   //***************************************************************************
   Res := FindFirst(SourceFileName,faAnyFile,Dir);
   if Res=0 Then
      Begin
        D1 := Dir.FindData.nFileSizeHigh;
        D2 := MAXDWORD;
        D2 := D2 + 1;
        D3 := Dir.FindData.nFileSizeLow;
        FS := (D1 * D2) + D3;
      End;
   FindClose(Dir);
   //***************************************************************************

   //***************************************************************************
   Repeat
     Abort  := False;
     Retry    := False;
     Try
       SF := TFileStream.Create(SourceFileName,fmOpenRead);
     Except
       Abort  := True;
       if Assigned(FOnError) Then FOnError(SourceFileName, TargetFileName,Abort,Retry,Ignore);
       if Abort Then
          Begin
            Result := Aborted;
            Exit;
          End;
     End;
   if FYeld Then Application.ProcessMessages;
   Until NOT Retry;
   //***************************************************************************

   //***************************************************************************
   Repeat
     Abort  := False;
     Retry  := False;
     Try
       TF := TFileStream.Create(TargetFileName,fmCreate or fmOpenWrite);
       SetFileAttributes(TargetFileName,FILE_ATTRIBUTE_ARCHIVE);
     Except
       if (NOT Ignore) Then
          Begin
            Abort  := True;
            if Assigned(FOnError) Then FOnError(SourceFileName, TargetFileName,Abort,Retry,Ignore);
            if Abort Then
               Begin
                 Result := Aborted;
                 Exit;
               End;
          End;
     End;
   if FYeld Then Application.ProcessMessages;
   Until NOT Retry;
   //***************************************************************************

   //***************************************************************************
   Repeat
     Repeat
       Abort  := False;
       Retry  := False;
       Try
         NR := SF.Read(BUF,FBufferSize);
         Inc(FBytesRead,NR);
       Except
         if (NOT Ignore) Then
            Begin
              Abort := True;
              if Assigned(FOnError) Then FOnError(SourceFileName, TargetFileName,Abort,Retry,Ignore);
              if Abort Then
                 Begin
                   Result := Aborted;
                   Exit;
                 End;
            End;
       End;
       if FYeld Then Application.ProcessMessages;
     Until NOT Retry;
     //***************************************************************************

     //***************************************************************************
     Repeat
       Abort  := False;
       Retry  := False;
       Try
         NW := TF.Write(BUF,NR);
         Inc(FBytesWritten, NW);
       Except
         if (NOT Ignore) Then
            Begin
              Abort := True;
              if Assigned(FOnError) Then FOnError(SourceFileName, TargetFileName,Abort,Retry,Ignore);
              if Abort Then
                 Begin
                   Result := Aborted;
                   Exit;
                 End;
            End;
       End;
     if FYeld Then Application.ProcessMessages;
     Until NOT Retry;
     //***************************************************************************

     //***************************************************************************
     Inc(FT,NW);
     If Assigned(FOnProgress) Then FOnProgress(SourceFileName, TargetFileName, FT, FS, Abort);
     //***************************************************************************

     if (NR > 0) And (NR <> NW) Then
        Begin
          if (NOT Ignore) Then
             Begin
               Abort := True;
               if Assigned(FOnError) Then FOnError(SourceFileName, TargetFileName,Abort,Retry,Ignore);
               if Abort Then
                  Begin
                    Result := Aborted;
                    Exit;
                  End;
             End;
        End;
   Until (NR=0);
   if Not Ignore Then Result := Copyed Else Result := Ignored;
 Finally
   if Abort  Then Result := Aborted;
   if Ignore Then Result := Ignored;
   If Assigned(SF) Then FreeAndNil(SF);
   If Assigned(TF) Then FreeAndNil(TF);
   SetLength(BUF,0);
 End;
 {$I-}
end;

function TKACopyFile.CopyDirectory(SourceDirName, TargetDirName: String; Var Abort : Boolean): TCopyResult;
Var
  Dir         : TSearchRec;
  CurrentDir  : TSearchRec;
  Res         : Integer;
  DoCopy      : Boolean;
  SDir        : String;
  TDir        : String;
  SF          : String;
  TF          : String;
begin
  Result := Aborted;
  if Abort Then Exit;
  //************************************************************** CREATE FOLDER
  SDir   := SourceDirName;
  TDir   := TargetDirName;
  Abort  := False;
  DoCopy := True;
  if Assigned(FOnBeforeCopyFolder) Then FOnBeforeCopyFolder(SDir,TDir,Abort,DoCopy);
  If Abort Then Exit;
  if Not DoCopy Then
     Begin
        Result := Skipped;
        Exit;
     End;
  if FWhatToCopy in [All, Folders] Then
     Begin
       if NOT DirectoryExists(ExtractFilePath(TDir)) Then ForceDirectories(ExtractFilePath(TDir));
       CreateDir(TDir);
     End;
  if FWhatToCopy = None Then Inc(FFoldersTotal);
  Inc(FFoldersCurrent);
  //************************************************************ COPY SUBFOLDERS
  Res := FindFirst(SDir+FDirWildCard,faAnyFile,Dir);
  Try
   While Res = 0 Do
    Begin
      if ((Dir.Attr And faDirectory) > 0) Then
         Begin
           if (Dir.Name <> '.') And (Dir.Name <> '..') Then
              Begin
                SF     := SDir+Dir.Name;
                if FWhatToCopy in [All, Folders] Then
                   TF := TrailingSlash(TDir)+Dir.Name
                Else
                   TF := UnTrailingSlash(TDir);
                if FWhatToCopy in [All, Folders, Files, None] Then
                   Begin
                      if FCopySubfolders Then
                         Begin
                           Result := Self.CopyDirectory(TrailingSlash(SF),TrailingSlash(TF), Abort);
                           if FYeld Then Application.ProcessMessages;
                           if Abort Then Exit;
                         End;
                   End;
              End;
         End;
      if FYeld Then Application.ProcessMessages;
      Res := FindNext(Dir);
    End;
    if Result <> Ignored Then Result := Copyed;
  Finally
    FindClose(Dir);
  End;
  //***************************************************************** COPY FILES
  if FWhatToCopy in [All, Files, None] Then
     Begin
       Res := FindFirst(SDir+FFileWildCard,faAnyFile,Dir);
       Try
         While Res = 0 Do
           Begin
             if ((Dir.Attr And faDirectory) = 0) Then
                  Begin
                    SF     := SDir+Dir.Name;
                    TF     := TDir+Dir.Name;
                    if FWhatToCopy in [All, Files] Then
                       Begin
                         Abort  := False;
                         DoCopy := True;
                         If Assigned(FOnBeforeCopy) Then FOnBeforeCopy(SF, TF, Abort, DoCopy);
                         if Abort Then Exit;
                         if DoCopy Then
                            Begin
                              Result := Self.DoCopyFile(SF,TF);
                              if FCopyDates             Then SetFileTime(TF, Dir);
                              if FCopyAttributes        Then SetAttributes(TF, Dir);
                              if FYeld                  Then Application.ProcessMessages;
                              If Assigned(FOnAfterCopy) Then FOnAfterCopy(SF,TF);
                              Inc(FFilesCurrent);
                              Inc(FBytesCurrent);
                            End;
                       End
                    Else
                       Begin
                         Inc(FBytesTotal,Dir.Size);
                         Inc(FFilesTotal);
                       End;
                    if Result=Aborted  Then
                       Begin
                         Abort := True;
                         Exit;
                       End;
                  End;
             if FYeld Then Application.ProcessMessages;
             Res := FindNext(Dir);
           End;
         if Result <> Ignored Then Result := Copyed;
       Finally
         FindClose(Dir);
       End;
    End;
  //************************************************** SETTING FOLDER ATTRIBUTES
  if FWhatToCopy in [All, Folders] Then
     Begin
       Res := FindFirst(UnTrailingSlash(SDir),faAnyFile,CurrentDir);
       Try
         if Res = 0 Then
            Begin
              if FCopyDates      Then SetFolderTime(TDir, CurrentDir);
              if FCopyAttributes Then SetAttributes(TDir, CurrentDir);
            End;
          if FYeld Then Application.ProcessMessages;
       Finally
           FindClose(CurrentDir);
       End;
     End;
  If Assigned(FOnAfterCopyFolder) Then FOnAfterCopyFolder(SDir,TDir);
end;

constructor TKACopyFile.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FWhatToCopy     := All;
  FCopyAttributes := True;
  FCopyDates      := True;
  FCopyDates      := True;
  FCopySubfolders := True;
  FBufferSize     := 100000;
  FSource         := '';
  FTarget         := '';
  FDirWildCard    := '*.*';
  FFileWildCard   := '*.*';
  FFailIfExists   := False;
  FRenameIfExists := False;
  FBytesRead      := 0;
  FBytesWritten   := 0;
  FBytesTotal     := 0;
  FFilesTotal     := 0;
  FFoldersTotal   := 0;
  FFilesCurrent   := 0;
  FBytesCurrent   := 0;
  FFoldersCurrent := 0;
  FYeld           := True;
  FActivate       := False;
end;

destructor TKACopyFile.Destroy;
begin

  inherited Destroy;
end;

Function TKACopyFile.TrailingSlash(DirectoryName:String):String;
Var
  L : Integer;
Begin
  Result := DirectoryName;
  L      := Length(Result);
  if (L > 0) And (Result[L] <> '\') Then Result := Result+'\';
End;

Function TKACopyFile.UnTrailingSlash(DirectoryName:String):String;
Var
  L : Integer;
Begin
  Result := DirectoryName;
  L      := Length(Result);
  if (L > 0) And (Result[L] = '\') Then Borland.Delphi.System.Delete(Result,L,1);
End;

Function TKACopyFile.GetUniqueName(FileName:String):String;
Var
  Path : String;
  Name : String;
  Ext  : String;
  Num  : Integer;
Begin
  Path := ExtractFilePath(FileName);
  Ext  := ExtractFileExt(FileName);
  Name := ExtractFileName(FileName);
  Name := Borland.Delphi.System.Copy(Name,1,Length(Name)-Length(Ext));
  Num  := 1;
  Repeat
    Result := Path+Name+'.'+Format('%.3d',[Num])+Ext;
    Inc(Num);
  Until NOT FileExists(Result);
End;

Function TKACopyFile.Run : TCopyResult;
Var
  Abort  : Boolean;
  DoCopy : Boolean;
  SF     : String;
  TF     : String;
begin
  FBytesRead      := 0;
  FBytesWritten   := 0;
  FFilesCurrent   := 0;
  FBytesCurrent   := 0;
  FFoldersCurrent := 0;
  Abort           := False;
  DoCopy          := True;
  Result          := Copyed;
  if FWhatToCopy = None Then
     Begin
       FFilesTotal    := 0;
       FFoldersTotal  := 0;
       FBytesTotal    := 0;
     End;
  //********************* Source is Directory
  if (DirectoryExists(FSource)) Then
     Begin
        //********************* Target is Existing File
        if (FileExists(FTarget)) Then
           Begin
             Raise Exception.Create('Cannot copy directory to file!');
           End
        Else
           Begin
             //********************* Target is Existing Directory
             If DirectoryExists(FTarget) Then
                Begin
                  Result := Self.CopyDirectory(TrailingSlash(FSource),TrailingSlash(FTarget), Abort);
                End
             Else
               //********************* Target is NonExisting Directory
               Begin
                  Result := Self.CopyDirectory(TrailingSlash(FSource),TrailingSlash(FTarget), Abort);
               End;
           End;
     End
  Else
  //********************* Source is File
  if (FileExists(FSource)) Then
     Begin
       //********************* Target is Existing File
       if (FileExists(FTarget)) Then
          Begin
            if FWhatToCopy in [All, Files] Then
               Begin
                 SF := FSource;
                 TF := FTarget;
                 If Assigned(FOnBeforeCopy) Then FOnBeforeCopy(SF, TF, Abort, DoCopy);
                 if Abort Then
                    Begin
                      Result := Aborted;
                      Exit;
                    End;
                 if DoCopy Then
                    Begin
                      Result := Self.DoCopyFile(SF,TF);
                      If Assigned(FOnAfterCopy) Then FOnAfterCopy(SF,TF);
                    End;
               End;
          End
       Else
          Begin
            //********************* Target is Existing Directory
            If DirectoryExists(FTarget) Then
               Begin
                 if FWhatToCopy in [All, Files] Then
                    Begin
                      SF := FSource;
                      TF := TrailingSlash(FTarget)+ExtractFileName(FSource);
                      If Assigned(FOnBeforeCopy) Then FOnBeforeCopy(SF, TF, Abort, DoCopy);
                      if Abort Then
                         Begin
                           Result := Aborted;
                           Exit;
                         End;
                      if DoCopy Then
                         Begin
                           Result := Self.DoCopyFile(SF,TF);
                           If Assigned(FOnAfterCopy) Then FOnAfterCopy(SF,TF);
                         End;
                    End;
               End
            Else
               //********************* Target is NonExisting File
               Begin
                 if FWhatToCopy in [All, Files] Then
                    Begin
                      SF := FSource;
                      TF := FTarget;
                      If Assigned(FOnBeforeCopy) Then FOnBeforeCopy(SF, TF, Abort, DoCopy);
                      if Abort Then
                         Begin
                           Result := Aborted;
                           Exit;
                         End;
                      if DoCopy Then
                         Begin
                           Result := Self.DoCopyFile(SF,TF);
                           If Assigned(FOnAfterCopy) Then FOnAfterCopy(SF,TF);
                         End;
                    End;
               End;
          End;
     End;
  if csDesigning in ComponentState Then  FActivate := False;
end;

Function TKACopyFile.CopyFile(Src:String; Dest:String):TCopyResult;
Begin
  FSource := Src;
  FTarget := Dest;
  Result  := Run;
End;

procedure TKACopyFile.SetActivate(const Value: Boolean);
begin
  FActivate := Value;
  if Value Then
     Begin
       Run;
     End;
end;

procedure TKACopyFile.SetBufferSize(const Value: Integer);
begin
  FBufferSize := Value;
end;

procedure TKACopyFile.SetCopySubfolders(const Value: Boolean);
begin
  FCopySubfolders := Value;
end;

procedure TKACopyFile.SetFailIfExists(const Value: Boolean);
begin
  FFailIfExists := Value;
  if csLoading in ComponentState Then Exit;
  FRenameIfExists := False;
end;

procedure TKACopyFile.SetOnAfterCopy(const Value: TOnAfterCopyEvent);
begin
  FOnAfterCopy := Value;
end;

procedure TKACopyFile.SetOnBeforeCopy(const Value: TOnBeforeCopyEvent);
begin
  FOnBeforeCopy := Value;
end;

procedure TKACopyFile.SetOnError(const Value: TOnErrorEvent);
begin
  FOnError := Value;
end;

procedure TKACopyFile.SetOnProgress(const Value: TOnProgressEvent);
begin
  FOnProgress := Value;
end;

procedure TKACopyFile.SetRenameIfExists(const Value: Boolean);
begin
  FRenameIfExists := Value;
  if csLoading in ComponentState Then Exit;
  FFailIfExists := False;
end;

procedure TKACopyFile.SetSource(const Value: String);
begin
  FSource := Value;
end;

procedure TKACopyFile.SetTarget(const Value: String);
begin
  FTarget := Value;
end;

procedure TKACopyFile.SetWhatToCopy(const Value: TCopyType);
begin
  FWhatToCopy := Value;
end;

procedure TKACopyFile.SetYeld(const Value: Boolean);
begin
  FYeld := Value;
end;

procedure TKACopyFile.SetDirWildCard(const Value: String);
begin
  FDirWildCard := Value;
end;

procedure TKACopyFile.SetFileWildCard(const Value: String);
begin
  FFileWildCard := Value;
end;

procedure TKACopyFile.SetCopyDates(const Value: Boolean);
begin
  FCopyDates := Value;
end;

procedure TKACopyFile.SetCopyAttributes(const Value: Boolean);
begin
  FCopyAttributes := Value;
end;


procedure TKACopyFile.SetOnAfterCopyFolder(const Value: TOnAfterCopyEvent);
begin
  FOnAfterCopyFolder := Value;
end;

procedure TKACopyFile.SetOnBeforeCopyFolder(
  const Value: TOnBeforeCopyEvent);
begin
  FOnBeforeCopyFolder := Value;
end;

procedure TKACopyFile.SetBytesTotal(const Value: Cardinal);
begin
  FBytesTotal := Value;
end;

procedure TKACopyFile.SetOnAutoReaname(const Value: TOnRenameEvent);
begin
  FOnAutoReaname := Value;
end;

procedure TKACopyFile.SetFilesTotal(const Value: Cardinal);
begin
  FFilesTotal := Value;
end;

procedure TKACopyFile.SetFoldersTotal(const Value: Cardinal);
begin
  FFoldersTotal := Value;
end;

procedure TKACopyFile.SetBytesRead(const Value: Cardinal);
begin
  FBytesRead := Value;
end;

procedure TKACopyFile.SetBytesWritten(const Value: Cardinal);
begin
  FBytesWritten := Value;
end;

procedure TKACopyFile.SetBytesCurrent(const Value: Cardinal);
begin
  FBytesCurrent := Value;
end;

procedure TKACopyFile.SetFilesCurrent(const Value: Cardinal);
begin
  FFilesCurrent := Value;
end;

procedure TKACopyFile.SetFoldersCurrent(const Value: Cardinal);
begin
  FFoldersCurrent := Value;
end;

end.
